@@ -321,16 +321,22 @@ QStringList KConfigPrivate::groupList(const QByteArray& group) const
     return groups.toList();
 }
 
+static bool isGroupOrSubGroupMatch(const QByteArray& potentialGroup, const QByteArray& group)
+{
+    if (!potentialGroup.startsWith(group)) {
+      return false;
+    }
+    return potentialGroup.length() == group.length() || potentialGroup[group.length()] == '\x1d';
+}
+
 // List all sub groups, including subsubgroups
 QSet<QByteArray> KConfigPrivate::allSubGroups(const QByteArray& parentGroup) const
 {
     QSet<QByteArray> groups;
-    QByteArray theGroup = parentGroup + '\x1d';
-    groups << parentGroup;
 
     for (KEntryMap::const_iterator entryMapIt = entryMap.begin(); entryMapIt != entryMap.end(); ++entryMapIt) {
         const KEntryKey& key = entryMapIt.key();
-        if (key.mKey.isNull() && key.mGroup.startsWith(theGroup)) {
+        if (key.mKey.isNull() && isGroupOrSubGroupMatch(key.mGroup, parentGroup)) {
             groups << key.mGroup;
         }
     }
@@ -339,12 +345,10 @@ QSet<QByteArray> KConfigPrivate::allSubGroups(const QByteArray& parentGroup) con
 
 bool KConfigPrivate::hasNonDeletedEntries(const QByteArray& group) const
 {
-    const QSet<QByteArray> allGroups = allSubGroups(group);
-
-    Q_FOREACH(const QByteArray& aGroup, allGroups) {
-        // Could be optimized, let's use the slow way for now
+    for (KEntryMap::const_iterator it = entryMap.begin(); it != entryMap.end(); ++it) {
+        const KEntryKey& key = it.key();
         // Check for any non-deleted entry
-        if (!keyListImpl(aGroup).isEmpty())
+        if (isGroupOrSubGroupMatch(key.mGroup, group) && !key.mKey.isNull() && !it->bDeleted)
             return true;
     }
     return false;
@@ -363,7 +367,7 @@ QStringList KConfigPrivate::keyListImpl(const QByteArray& theGroup) const
         QSet<QString> tmp;
         for (; it != theEnd && it.key().mGroup == theGroup; ++it) {
             const KEntryKey& key = it.key();
-            if (key.mGroup == theGroup && !key.mKey.isNull() && !it->bDeleted)
+            if (!key.mKey.isNull() && !it->bDeleted)
                 tmp << QString::fromUtf8(key.mKey);
         }
         keys = tmp.toList();
@@ -492,7 +496,7 @@ void KConfig::markAsClean()
         it->bDirty = false;
 }
 
-bool KConfig::isDirty() const // only exists for the unittest
+bool KConfig::isDirty() const
 {
     Q_D(const KConfig);
     return d->bDirty;
@@ -566,7 +570,11 @@ void KConfigPrivate::changeFileName(const QString& name, const char* type)
         return;
     }
 
+#ifndef Q_OS_WIN
     bSuppressGlobal = (file == sGlobalFileName);
+#else
+    bSuppressGlobal = (file.compare(sGlobalFileName, Qt::CaseInsensitive) == 0);
+#endif
 
     if (bDynamicBackend || !mBackend) // allow dynamic changing of backend
         mBackend = KConfigBackend::create(componentData, file);
@@ -622,7 +630,11 @@ void KConfigPrivate::parseGlobalFiles()
     const QByteArray utf8Locale = locale.toUtf8();
     foreach(const QString& file, globalFiles) {
         KConfigBackend::ParseOptions parseOpts = KConfigBackend::ParseGlobal|KConfigBackend::ParseExpansions;
+#ifndef Q_OS_WIN
         if (file != sGlobalFileName)
+#else
+        if (file.compare(sGlobalFileName, Qt::CaseInsensitive) != 0)
+#endif
             parseOpts |= KConfigBackend::ParseDefaults;
 
         KSharedPtr<KConfigBackend> backend = KConfigBackend::create(componentData, file);
@@ -657,7 +669,11 @@ void KConfigPrivate::parseConfigFiles()
 
         const QByteArray utf8Locale = locale.toUtf8();
         foreach(const QString& file, files) {
+#ifndef Q_OS_WIN
             if (file == mBackend->filePath()) {
+#else
+            if (file.compare(mBackend->filePath(), Qt::CaseInsensitive) == 0) {
+#endif
                 switch (mBackend->parseConfig(utf8Locale, entryMap, KConfigBackend::ParseExpansions)) {
                 case KConfigBackend::ParseOk:
                     break;
