@@ -24,7 +24,10 @@
 #include <kdemacros.h>
 
 #ifndef KCMUTILS_EXPORT
-# if defined(MAKE_KCMUTILS_LIB)
+# if defined(KDELIBS_STATIC_LIBS)
+   /* No export/import for static libraries */
+#  define KCMUTILS_EXPORT
+# elif defined(MAKE_KCMUTILS_LIB)
    /* We are building this library */ 
 #  define KCMUTILS_EXPORT KDE_EXPORT
 # else
